@@ -86,14 +86,16 @@ gst_ffmpeg_set_palette (GstCaps *caps, AVCodecContext *context)
     gst_caps_new_simple (mimetype,			      	\
 	"width",     G_TYPE_INT,   context->width,	      	\
 	"height",    G_TYPE_INT,   context->height,	  	\
-	"framerate", G_TYPE_DOUBLE, 1. * context->frame_rate /  \
-				   context->frame_rate_base,    \
+	"framerate", G_TYPE_DOUBLE, (double) 1. *		\
+				context->time_base.den /	\
+					context->time_base.num,	\
 	__VA_ARGS__, NULL)  					\
     :	  							\
     gst_caps_new_simple (mimetype,			      	\
 	"width",     GST_TYPE_INT_RANGE, 16, 4096,      	\
 	"height",    GST_TYPE_INT_RANGE, 16, 4096,	      	\
-	"framerate", GST_TYPE_DOUBLE_RANGE, 0., G_MAXDOUBLE,	\
+	"framerate", GST_TYPE_DOUBLE_RANGE, (double) 0., 	\
+						G_MAXDOUBLE,	\
 	__VA_ARGS__, NULL)
 
 /* same for audio - now with channels/sample rate
@@ -107,6 +109,8 @@ gst_ffmpeg_set_palette (GstCaps *caps, AVCodecContext *context)
 	__VA_ARGS__, NULL)					\
     :								\
     gst_caps_new_simple (mimetype,	      			\
+        "rate", GST_TYPE_INT_RANGE, 8000, 96000,		\
+	"channels", GST_TYPE_INT_RANGE, 1, 2,			\
 	__VA_ARGS__, NULL)
 
 /* Convert a FFMPEG codec ID and optional AVCodecContext
@@ -326,11 +330,7 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
 
     case CODEC_ID_SVQ3:
       caps = GST_FF_VID_CAPS_NEW ("video/x-svq",
-          "svqversion", G_TYPE_INT, 3,
-          "halfpel_flag", GST_TYPE_INT_RANGE, 0, 1,
-          "thirdpel_flag", GST_TYPE_INT_RANGE, 0, 1,
-          "low_delay", GST_TYPE_INT_RANGE, 0, 1,
-          "unknown_svq3_flag", GST_TYPE_INT_RANGE, 0, 1, NULL);
+          "svqversion", G_TYPE_INT, 3, NULL);
       break;
 
     case CODEC_ID_DVAUDIO:
@@ -393,6 +393,11 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
           "indeoversion", G_TYPE_INT, 3, NULL);
       break;
 
+    case CODEC_ID_INDEO2:
+      caps = GST_FF_VID_CAPS_NEW ("video/x-indeo",
+          "indeoversion", G_TYPE_INT, 2, NULL);
+      break;
+
     case CODEC_ID_VP3:
       caps = GST_FF_VID_CAPS_NEW ("video/x-vp3", NULL);
       break;
@@ -427,6 +432,7 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
           "wcversion", G_TYPE_INT, 3 - CODEC_ID_XAN_WC3 + codec_id, NULL);
       break;
 
+    case CODEC_ID_FRAPS:
     case CODEC_ID_VCR1:
     case CODEC_ID_CLJR:
     case CODEC_ID_MDEC:
@@ -461,6 +467,12 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
 	  "msvideoversion", G_TYPE_INT, 1, NULL);
       break;
 
+    case CODEC_ID_WMV3:
+    case CODEC_ID_VC9:
+      caps = GST_FF_VID_CAPS_NEW ("video/x-wmv",
+          "wmvversion", G_TYPE_INT, 3, NULL);
+      break;
+
     case CODEC_ID_WS_VQA:
     case CODEC_ID_IDCIN:
     case CODEC_ID_8BPS:
@@ -488,6 +500,12 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
     case CODEC_ID_PGMYUV:
     case CODEC_ID_PAM:
     case CODEC_ID_FFVHUFF:
+    case CODEC_ID_LOCO:
+    case CODEC_ID_WNV1:
+    case CODEC_ID_AASC:
+    case CODEC_ID_MP3ADU:
+    case CODEC_ID_MP3ON4:
+    case CODEC_ID_WESTWOOD_SND1:
       buildcaps = TRUE;
       break;
 
@@ -573,6 +591,8 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
     case CODEC_ID_ADPCM_EA:
     case CODEC_ID_ADPCM_G726:
     case CODEC_ID_ADPCM_CT:
+    case CODEC_ID_ADPCM_SWF:
+    case CODEC_ID_ADPCM_YAMAHA:
       do {
         gchar *layout = NULL;
 
@@ -616,6 +636,12 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
           case CODEC_ID_ADPCM_CT:
             layout = "ct";
             break;
+          case CODEC_ID_ADPCM_SWF:
+            layout = "swf";
+            break;
+          case CODEC_ID_ADPCM_YAMAHA:
+            layout = "yamaha";
+            break;
           default:
             g_assert (0);       /* don't worry, we never get here */
             break;
@@ -687,6 +713,18 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
       } while (0);
       break;
 
+    case CODEC_ID_SHORTEN:
+      caps = gst_caps_new_simple ("audio/x-shorten", NULL);
+      break;
+
+    case CODEC_ID_ALAC:
+      caps = GST_FF_AUD_CAPS_NEW ("audio/x-alac", NULL);
+      if (context) {
+        gst_caps_set_simple (caps,
+            "samplesize", G_TYPE_INT, context->bits_per_sample, NULL);
+      }
+      break;
+
     case CODEC_ID_FLAC:
       /* Note that ffmpeg has no encoder yet, but just for safety. In the
        * encoder case, we want to add things like samplerate, channels... */
@@ -695,6 +733,11 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
       }
       break;
 
+    case CODEC_ID_DVD_SUBTITLE:
+    case CODEC_ID_DVB_SUBTITLE:
+      caps = NULL;
+      break;
+
     default:
       g_warning ("Unknown codec ID %d, please add here", codec_id);
       break;
@@ -773,8 +816,9 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
   int bpp = 0, depth = 0, endianness = 0;
   gulong g_mask = 0, r_mask = 0, b_mask = 0, a_mask = 0;
   guint32 fmt = 0;
-
+  
   switch (pix_fmt) {
+    case PIX_FMT_YUVJ420P:
     case PIX_FMT_YUV420P:
       fmt = GST_MAKE_FOURCC ('I', '4', '2', '0');
       break;
@@ -795,11 +839,13 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       g_mask = 0x00ff00;
       b_mask = 0xff0000;
       break;
+    case PIX_FMT_YUVJ422P:
     case PIX_FMT_YUV422P:
       fmt = GST_MAKE_FOURCC ('Y', '4', '2', 'B');
       break;
+    case PIX_FMT_YUVJ444P:
     case PIX_FMT_YUV444P:
-      /* .. */
+      fmt = GST_MAKE_FOURCC ('Y', '4', '4', '4');
       break;
     case PIX_FMT_RGBA32:
       bpp = 32;
@@ -842,35 +888,43 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       bpp = depth = 8;
       endianness = G_BYTE_ORDER;
       break;
+    case PIX_FMT_GRAY8:
+      bpp = depth = 8;
+      caps = GST_FF_VID_CAPS_NEW ("video/x-raw-gray",
+				  "bpp", G_TYPE_INT, bpp,
+				  "depth", G_TYPE_INT, depth, NULL);
+      break;
     default:
       /* give up ... */
       break;
   }
-
-  if (bpp != 0) {
-    if (r_mask != 0) {
-      caps = GST_FF_VID_CAPS_NEW ("video/x-raw-rgb",
-          "bpp", G_TYPE_INT, bpp,
-          "depth", G_TYPE_INT, depth,
-          "red_mask", G_TYPE_INT, r_mask,
-          "green_mask", G_TYPE_INT, g_mask,
-          "blue_mask", G_TYPE_INT, b_mask,
-          "endianness", G_TYPE_INT, endianness, NULL);
-      if (a_mask) {
-        gst_caps_set_simple (caps, "alpha_mask", G_TYPE_INT, a_mask, NULL);
-      }
-    } else {
-      caps = GST_FF_VID_CAPS_NEW ("video/x-raw-rgb",
-          "bpp", G_TYPE_INT, bpp,
-          "depth", G_TYPE_INT, depth,
-          "endianness", G_TYPE_INT, endianness, NULL);
-      if (context) {
-        gst_ffmpeg_set_palette (caps, context);
+  
+  if (caps == NULL) {
+    if (bpp != 0) {
+      if (r_mask != 0) {
+	caps = GST_FF_VID_CAPS_NEW ("video/x-raw-rgb",
+				    "bpp", G_TYPE_INT, bpp,
+				    "depth", G_TYPE_INT, depth,
+				    "red_mask", G_TYPE_INT, r_mask,
+				    "green_mask", G_TYPE_INT, g_mask,
+				    "blue_mask", G_TYPE_INT, b_mask,
+				    "endianness", G_TYPE_INT, endianness, NULL);
+	if (a_mask) {
+	  gst_caps_set_simple (caps, "alpha_mask", G_TYPE_INT, a_mask, NULL);
+	}
+      } else {
+	caps = GST_FF_VID_CAPS_NEW ("video/x-raw-rgb",
+				    "bpp", G_TYPE_INT, bpp,
+				    "depth", G_TYPE_INT, depth,
+				    "endianness", G_TYPE_INT, endianness, NULL);
+	if (context) {
+	  gst_ffmpeg_set_palette (caps, context);
+	}
       }
+    } else if (fmt) {
+      caps = GST_FF_VID_CAPS_NEW ("video/x-raw-yuv",
+				  "format", GST_TYPE_FOURCC, fmt, NULL);
     }
-  } else if (fmt) {
-    caps = GST_FF_VID_CAPS_NEW ("video/x-raw-yuv",
-        "format", GST_TYPE_FOURCC, fmt, NULL);
   }
 
   if (caps != NULL) {
@@ -1047,8 +1101,8 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
   gst_structure_get_int (structure, "bpp", &context->bits_per_sample);
 
   if (gst_structure_get_double (structure, "framerate", &fps)) {
-    context->frame_rate = fps * DEFAULT_FRAME_RATE_BASE;
-    context->frame_rate_base = DEFAULT_FRAME_RATE_BASE;
+    context->time_base.den = fps * DEFAULT_FRAME_RATE_BASE;
+    context->time_base.num = DEFAULT_FRAME_RATE_BASE;
   }
 
   if (!raw)
@@ -1195,35 +1249,38 @@ gst_ffmpeg_caps_with_codecid (enum CodecID codec_id,
       } while (0);
       break;
 
-    case CODEC_ID_SVQ3:
-      do {
-        gint halfpel_flag, thirdpel_flag, low_delay, unknown_svq3_flag;
-        guint16 flags;
+     case CODEC_ID_SVQ3:
+       /* FIXME: this is a workaround for older gst-plugins releases
+        * (<= 0.8.9). This should be removed at some point, because
+        * it causes wrong decoded frame order. */
+       if (!context->extradata) {
+         gint halfpel_flag, thirdpel_flag, low_delay, unknown_svq3_flag;
+         guint16 flags;
+  	 
+         if (gst_structure_get_int (str, "halfpel_flag", &halfpel_flag) ||
+             gst_structure_get_int (str, "thirdpel_flag", &thirdpel_flag) ||
+             gst_structure_get_int (str, "low_delay", &low_delay) ||
+             gst_structure_get_int (str, "unknown_svq3_flag",
+                 &unknown_svq3_flag)) {
+           context->extradata = (guint8 *) av_mallocz (0x64);
+           g_stpcpy (context->extradata, "SVQ3");
+           flags = 1 << 3;
+           flags |= low_delay;
+           flags = flags << 2;
+           flags |= unknown_svq3_flag;
+           flags = flags << 6;
+           flags |= halfpel_flag;
+           flags = flags << 1;
+           flags |= thirdpel_flag;
+           flags = flags << 3;
 
-        if (gst_structure_get_int (str, "halfpel_flag", &halfpel_flag) ||
-            gst_structure_get_int (str, "thirdpel_flag", &thirdpel_flag) ||
-            gst_structure_get_int (str, "low_delay", &low_delay) ||
-            gst_structure_get_int (str, "unknown_svq3_flag",
-		&unknown_svq3_flag)) {
-          context->extradata = (guint8 *) av_mallocz (0x64);
-          g_stpcpy (context->extradata, "SVQ3");
-          flags = 1 << 3;
-          flags |= low_delay;
-          flags = flags << 2;
-          flags |= unknown_svq3_flag;
-          flags = flags << 6;
-          flags |= halfpel_flag;
-          flags = flags << 1;
-          flags |= thirdpel_flag;
-          flags = flags << 3;
-
-          flags = GUINT16_FROM_LE (flags);
-
-          memcpy (context->extradata + 0x62, &flags, 2);
-          context->extradata_size = 0x64;
-        }
-      } while (0);
-      break;
+           flags = GUINT16_FROM_LE (flags);
+ 
+           memcpy (context->extradata + 0x62, &flags, 2);
+           context->extradata_size = 0x64;
+         }
+       }
+       break;
 
     case CODEC_ID_MSRLE:
     case CODEC_ID_QTRLE:
@@ -1245,6 +1302,10 @@ gst_ffmpeg_caps_with_codecid (enum CodecID codec_id,
       } while (0);
       break;
 
+    case CODEC_ID_ALAC:
+      gst_structure_get_int (str, "samplesize", &context->bits_per_sample);
+      break;
+
     default:
       break;
   }
@@ -1313,6 +1374,8 @@ gst_ffmpeg_formatid_to_caps (const gchar * format_name)
     caps = gst_caps_new_simple ("application/x-id3", NULL);
   } else if (!strcmp (format_name, "flic")) {
     caps = gst_caps_new_simple ("video/x-fli", NULL);
+  } else if (!strcmp (format_name, "flv")) {
+    caps = gst_caps_new_simple ("video/x-flv", NULL);
   } else {
     gchar *name;
 
@@ -1325,6 +1388,42 @@ gst_ffmpeg_formatid_to_caps (const gchar * format_name)
   return caps;
 }
 
+gboolean
+gst_ffmpeg_formatid_get_codecids (const gchar *format_name,
+    enum CodecID ** video_codec_list, enum CodecID ** audio_codec_list)
+{
+  if (!strcmp (format_name, "mp4")) {
+    static enum CodecID mp4_video_list[] = { CODEC_ID_MPEG4, CODEC_ID_H264, CODEC_ID_NONE };
+    static enum CodecID mp4_audio_list[] = { CODEC_ID_AAC, CODEC_ID_NONE };
+
+    *video_codec_list = mp4_video_list;
+    *audio_codec_list = mp4_audio_list;
+  } else if (!strcmp (format_name, "mpeg")) {
+    static enum CodecID mpeg_video_list[] = { CODEC_ID_MPEG1VIDEO, CODEC_ID_NONE };
+    static enum CodecID mpeg_audio_list[] = { CODEC_ID_MP2, CODEC_ID_NONE };
+
+    *video_codec_list = mpeg_video_list;
+    *audio_codec_list = mpeg_audio_list;
+  } else if (!strcmp (format_name, "vob")) {
+    static enum CodecID vob_video_list[] = { CODEC_ID_MPEG2VIDEO, CODEC_ID_NONE };
+    static enum CodecID vob_audio_list[] = { CODEC_ID_MP2, CODEC_ID_AC3, CODEC_ID_NONE };
+
+    *video_codec_list = vob_video_list;
+    *audio_codec_list = vob_audio_list;
+  } else if (!strcmp (format_name, "flv")) {
+    static enum CodecID flv_video_list[] = { CODEC_ID_FLV1, CODEC_ID_NONE };
+    static enum CodecID flv_audio_list[] = { CODEC_ID_MP3, CODEC_ID_NONE };
+
+    *video_codec_list = flv_video_list;
+    *audio_codec_list = flv_audio_list;
+  } else {
+    GST_WARNING ("Format %s not found", format_name);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /* Convert a GstCaps to a FFMPEG codec ID. Size et all
  * are omitted, that can be queried by the user itself,
  * we're not eating the GstCaps or anything
@@ -1398,7 +1497,7 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
   } else if (!strcmp (mimetype, "video/x-dv")) {
     gboolean sys_strm;
 
-    if (!gst_structure_get_boolean (structure, "systemstream", &sys_strm) &&
+    if (gst_structure_get_boolean (structure, "systemstream", &sys_strm) &&
         !sys_strm) {
       id = CODEC_ID_DVVIDEO;
       video = TRUE;
@@ -1419,8 +1518,8 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
     gboolean sys_strm;
     gint mpegversion;
 
-    if (!gst_structure_get_boolean (structure, "systemstream", &sys_strm) &&
-        !gst_structure_get_int (structure, "mpegversion", &mpegversion) &&
+    if (gst_structure_get_boolean (structure, "systemstream", &sys_strm) &&
+        gst_structure_get_int (structure, "mpegversion", &mpegversion) &&
         !sys_strm) {
       switch (mpegversion) {
         case 1:
@@ -1453,6 +1552,9 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
         case 2:
           id = CODEC_ID_WMV2;
           break;
+        case 3:
+          id = CODEC_ID_WMV3;
+          break;
       }
     }
     if (id != CODEC_ID_NONE)
@@ -1468,7 +1570,7 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
       switch (mpegversion) {
         case 2:                /* ffmpeg uses faad for both... */
         case 4:
-          id = CODEC_ID_MPEG4AAC;
+          id = CODEC_ID_AAC;
           break;
         case 1:
           if (gst_structure_get_int (structure, "layer", &layer)) {
@@ -1564,10 +1666,17 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
   } else if (!strcmp (mimetype, "video/x-indeo")) {
     gint indeoversion = 0;
 
-    if (gst_structure_get_int (structure, "indeoversion", &indeoversion) &&
-        indeoversion == 3) {
-      id = CODEC_ID_INDEO3;
-      video = TRUE;
+    if (gst_structure_get_int (structure, "indeoversion", &indeoversion)) {
+      switch (indeoversion) {
+        case 3:
+          id = CODEC_ID_INDEO3;
+          break;
+        case 2:
+          id = CODEC_ID_INDEO2;
+          break;
+      }
+      if (id != CODEC_ID_NONE)
+        video = TRUE;
     }
   } else if (!strcmp (mimetype, "video/x-divx")) {
     gint divxversion = 0;
@@ -1631,6 +1740,10 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
       id = CODEC_ID_ADPCM_G726;
     } else if (!strcmp (layout, "ct")) {
       id = CODEC_ID_ADPCM_CT;
+    } else if (!strcmp (layout, "swf")) {
+      id = CODEC_ID_ADPCM_SWF;
+    } else if (!strcmp (layout, "yamaha")) {
+      id = CODEC_ID_ADPCM_YAMAHA;
     }
     if (id != CODEC_ID_NONE)
       audio = TRUE;
@@ -1657,6 +1770,12 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
   } else if (!strcmp (mimetype, "audio/x-flac")) {
     id = CODEC_ID_FLAC;
     audio = TRUE;
+  } else if (!strcmp (mimetype, "audio/x-shorten")) {
+    id = CODEC_ID_SHORTEN;
+    audio = TRUE;
+  } else if (!strcmp (mimetype, "audio/x-alac")) {
+    id = CODEC_ID_ALAC;
+    audio = TRUE;
   } else if (!strcmp (mimetype, "video/x-cinepak")) {
     id = CODEC_ID_CINEPAK;
     video = TRUE;
@@ -1722,20 +1841,29 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
   } else if (!strcmp (mimetype, "audio/x-amrwb")) {
     id = CODEC_ID_AMR_WB;
     audio = TRUE;
-  } else if (!strncmp (mimetype, "audio/x-gst_ff-", 15) ||
-      !strncmp (mimetype, "video/x-gst_ff-", 15)) {
+  } else if (!strcmp (mimetype, "video/x-h264")) {
+    id = CODEC_ID_H264;
+    video = TRUE;
+  } else if (!strncmp (mimetype, "audio/x-gst_ff-", 15) ) {
     gchar ext[16];
     AVCodec *codec;
-
     if (strlen (mimetype) <= 30 &&
-        sscanf (mimetype, "%*s/x-gst_ff-%s", ext) == 1) {
+        sscanf (mimetype, "audio/x-gst_ff-%s", ext) == 1) {
       if ((codec = avcodec_find_decoder_by_name (ext)) ||
           (codec = avcodec_find_encoder_by_name (ext))) {
         id = codec->id;
-        if (mimetype[0] == 'v')
-          video = TRUE;
-        else if (mimetype[0] == 'a')
-          audio = TRUE;
+	audio = TRUE;
+      }
+    }
+  } else if (!strncmp (mimetype, "video/x-gst_ff-", 15)) {
+    gchar ext[16];
+    AVCodec *codec;
+    if (strlen (mimetype) <= 30 &&
+        sscanf (mimetype, "video/x-gst_ff-%s", ext) == 1) {
+      if ((codec = avcodec_find_decoder_by_name (ext)) ||
+          (codec = avcodec_find_encoder_by_name (ext))) {
+        id = codec->id;
+        video = TRUE;
       }
     }
   }
@@ -1828,6 +1956,12 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_WMV2:
       name = "Windows Media Video v8";
       break;
+    case CODEC_ID_WMV3:
+      name = "Windows Media Video v9";
+      break;
+    case CODEC_ID_VC9:
+      name = "Microsoft Video Codec v1";
+      break;
     case CODEC_ID_H263P:
       name = "H.263 (P) video";
       break;
@@ -1873,6 +2007,9 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_INDEO3:
       name = "Indeo-3 video";
       break;
+    case CODEC_ID_INDEO2:
+      name = "Indeo=2 video";
+      break;
     case CODEC_ID_VP3:
       name = "VP3 video";
       break;
@@ -2009,6 +2146,24 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_FFVHUFF:
       name = "FFMPEG non-compliant Huffyuv video";
       break;
+    case CODEC_ID_LOCO:
+      name = "LOCO video";
+      break;
+    case CODEC_ID_WNV1:
+      name = "Winnov video 1";
+      break;
+    case CODEC_ID_AASC:
+      name = "Autodesk RLE video";
+      break;
+    case CODEC_ID_MP3ADU:
+      name = "ADU-formatted MPEG-1 layer 3 audio";
+      break;
+    case CODEC_ID_MP3ON4:
+      name = "MP3ON4";
+      break;
+    case CODEC_ID_WESTWOOD_SND1:
+      name = "Westwood Sound-1";
+      break;
     case CODEC_ID_PCM_MULAW:
       name = "Mu-law audio";
       break;
@@ -2054,6 +2209,12 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_ADPCM_CT:
       name = "CT ADPCM";
       break;
+    case CODEC_ID_ADPCM_SWF:
+      name = "Shockwave ADPCM";
+      break;
+    case CODEC_ID_ADPCM_YAMAHA:
+      name = "Yamaha ADPCM";
+      break;
     case CODEC_ID_RA_144:
       name = "Realaudio 14k4bps";
       break;
@@ -2075,6 +2236,21 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_FLAC:
       name = "FLAC lossless audio";
       break;
+    case CODEC_ID_SHORTEN:
+      name = "Shorten lossless audio";
+      break;
+    case CODEC_ID_ALAC:
+      name = "Apple lossless audio";
+      break;
+    case CODEC_ID_DVD_SUBTITLE:
+      name = "DVD subtitle";
+      break;
+    case CODEC_ID_DVB_SUBTITLE:
+      name = "DVB subtitle";
+      break;
+    case CODEC_ID_FRAPS:
+      name = "FRAPS video";
+      break;
     default:
       GST_WARNING ("Unknown codecID 0x%x", codec_id);
       break;
@@ -2403,7 +2579,6 @@ int
 gst_ffmpeg_img_convert (AVPicture * dst, int dst_pix_fmt,
     const AVPicture * src, int src_pix_fmt, int src_width, int src_height)
 {
-  int i;
   PixFmtInfo *pf = &pix_fmt_info[src_pix_fmt];
 
   pf = &pix_fmt_info[src_pix_fmt];
@@ -2422,6 +2597,3 @@ gst_ffmpeg_img_convert (AVPicture * dst, int dst_pix_fmt,
   }
   return img_convert (dst, dst_pix_fmt, src, src_pix_fmt, src_width, src_height);
 }
-
-
-
