@@ -35,6 +35,8 @@
 #include "gstffmpeg.h"
 #include "gstffmpegcodecmap.h"
 
+//#define FORCE_OUR_GET_BUFFER
+
 typedef struct _GstFFMpegDec GstFFMpegDec;
 
 struct _GstFFMpegDec
@@ -52,17 +54,21 @@ struct _GstFFMpegDec
   union {
     struct {
       gint width, height, fps, fps_base;
+      enum PixelFormat pix_fmt;
     } video;
     struct {
       gint channels, samplerate;
     } audio;
   } format;
+  gboolean waiting_for_key;
   guint64 next_ts;
 
   /* parsing */
   AVCodecParserContext *pctx;
   GstBuffer *pcache;
 
+  GstBuffer *last_buffer;
+
   GValue *par;		/* pixel aspect ratio of incoming data */
 
   gint hurry_up, lowres;
@@ -127,13 +133,13 @@ static void gst_ffmpegdec_set_property (GObject * object,
 static void gst_ffmpegdec_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
-#if 0
+static gboolean gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec);
+
 /* some sort of bufferpool handling, but different */
 static int gst_ffmpegdec_get_buffer (AVCodecContext * context,
     AVFrame * picture);
 static void gst_ffmpegdec_release_buffer (AVCodecContext * context,
     AVFrame * picture);
-#endif
 
 static GstElementClass *parent_class = NULL;
 
@@ -273,8 +279,11 @@ gst_ffmpegdec_init (GstFFMpegDec * ffmpegdec)
   ffmpegdec->pcache = NULL;
   ffmpegdec->par = NULL;
   ffmpegdec->opened = FALSE;
+  ffmpegdec->waiting_for_key = FALSE;
   ffmpegdec->hurry_up = ffmpegdec->lowres = 0;
 
+  ffmpegdec->last_buffer = NULL;
+
   GST_FLAG_SET (ffmpegdec, GST_ELEMENT_EVENT_AWARE);
 }
 
@@ -395,7 +404,8 @@ gst_ffmpegdec_open (GstFFMpegDec *ffmpegdec)
    * of $(see mpeg4)... */
   if (oclass->in_plugin->id != CODEC_ID_MPEG4 &&
       oclass->in_plugin->id != CODEC_ID_MJPEG &&
-      oclass->in_plugin->id != CODEC_ID_MP3) {
+      oclass->in_plugin->id != CODEC_ID_MP3 &&
+      oclass->in_plugin->id != CODEC_ID_H264) {
     ffmpegdec->pctx = av_parser_init (oclass->in_plugin->id);
   }
 
@@ -405,6 +415,7 @@ gst_ffmpegdec_open (GstFFMpegDec *ffmpegdec)
       ffmpegdec->format.video.height = 0;
       ffmpegdec->format.video.fps = 0;
       ffmpegdec->format.video.fps_base = 0;
+      ffmpegdec->format.video.pix_fmt = PIX_FMT_NB;
       break;
     case CODEC_TYPE_AUDIO:
       ffmpegdec->format.audio.samplerate = 0;
@@ -414,6 +425,8 @@ gst_ffmpegdec_open (GstFFMpegDec *ffmpegdec)
       break;
   }
   ffmpegdec->next_ts = 0;
+  
+  ffmpegdec->last_buffer = NULL;
 
   return TRUE;
 }
@@ -433,15 +446,17 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
   /* set defaults */
   avcodec_get_context_defaults (ffmpegdec->context);
 
-#if 0
-  /* set buffer functions */
+  /* set buffer functions */  
   ffmpegdec->context->get_buffer = gst_ffmpegdec_get_buffer;
-  ffmpegdec->context->release_buffer = gst_ffmpegdec_release_buffer;
-#endif
-
+  ffmpegdec->context->release_buffer = gst_ffmpegdec_release_buffer;      
+  
   /* get size and so */
   gst_ffmpeg_caps_with_codecid (oclass->in_plugin->id,
       oclass->in_plugin->type, caps, ffmpegdec->context);
+  if (!ffmpegdec->context->time_base.den) {
+    ffmpegdec->context->time_base.num = 1;
+    ffmpegdec->context->time_base.den = 25;
+  }
 
   /* get pixel aspect ratio if it's set */
   structure = gst_caps_get_structure (caps, 0);
@@ -476,23 +491,51 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
   return GST_PAD_LINK_OK;
 }
 
-#if 0
 static int
 gst_ffmpegdec_get_buffer (AVCodecContext * context, AVFrame * picture)
 {
   GstBuffer *buf = NULL;
   gulong bufsize = 0;
+  GstFFMpegDec *ffmpegdec = (GstFFMpegDec *) context->opaque; 
+  int width  = context->width;
+  int height = context->height;
 
   switch (context->codec_type) {
     case CODEC_TYPE_VIDEO:
+      
+      avcodec_align_dimensions(context, &width, &height);
+      
       bufsize = avpicture_get_size (context->pix_fmt,
-          context->width, context->height);
-      buf = gst_buffer_new_and_alloc (bufsize);
+				    width, height);
+      
+      if((width != context->width) || (height != context->height)) {
+#ifdef FORCE_OUR_GET_BUFFER
+	context->width = width;
+	context->height = height;
+#else	
+	/* revert to ffmpeg's default functions */
+	ffmpegdec->context->get_buffer = avcodec_default_get_buffer;
+	ffmpegdec->context->release_buffer = avcodec_default_release_buffer;       	
+
+	return avcodec_default_get_buffer(context, picture);
+#endif
+	
+      } 
+      
+      if (!gst_ffmpegdec_negotiate (ffmpegdec)) {
+	GST_ELEMENT_ERROR (ffmpegdec, CORE, NEGOTIATION, (NULL),
+			   ("Failed to link ffmpeg decoder to next element"));
+	return avcodec_default_get_buffer(context, picture);
+      }
+      
+      buf = gst_pad_alloc_buffer (ffmpegdec->srcpad, GST_BUFFER_OFFSET_NONE, bufsize);
+      ffmpegdec->last_buffer = buf;
+      
       gst_ffmpeg_avpicture_fill ((AVPicture *) picture,
           GST_BUFFER_DATA (buf),
           context->pix_fmt, context->width, context->height);
       break;
-
+      
     case CODEC_TYPE_AUDIO:
     default:
       g_assert (0);
@@ -507,7 +550,7 @@ gst_ffmpegdec_get_buffer (AVCodecContext * context, AVFrame * picture)
    * so that we don't need to copy data */
   picture->type = FF_BUFFER_TYPE_USER;
   picture->age = G_MAXINT;
-  picture->base[0] = (int8_t *) buf;
+  picture->opaque = buf;
   gst_buffer_ref (buf);
 
   return 0;
@@ -517,8 +560,14 @@ static void
 gst_ffmpegdec_release_buffer (AVCodecContext * context, AVFrame * picture)
 {
   gint i;
-  GstBuffer *buf = GST_BUFFER (picture->base[0]);
+  GstBuffer *buf = GST_BUFFER (picture->opaque);
+  GstFFMpegDec *ffmpegdec = (GstFFMpegDec *) context->opaque; 
+  
+  g_return_if_fail (buf != NULL);
+  g_return_if_fail (picture->type == FF_BUFFER_TYPE_USER);
 
+  if (buf == ffmpegdec->last_buffer)
+    ffmpegdec->last_buffer = NULL;
   gst_buffer_unref (buf);
 
   /* zero out the reference in ffmpeg */
@@ -527,7 +576,6 @@ gst_ffmpegdec_release_buffer (AVCodecContext * context, AVFrame * picture)
     picture->linesize[i] = 0;
   }
 }
-#endif
 
 static gboolean
 gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec)
@@ -540,19 +588,22 @@ gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec)
     case CODEC_TYPE_VIDEO:
       if (ffmpegdec->format.video.width == ffmpegdec->context->width &&
           ffmpegdec->format.video.height == ffmpegdec->context->height &&
-          ffmpegdec->format.video.fps == ffmpegdec->context->frame_rate &&
+          ffmpegdec->format.video.fps == ffmpegdec->context->time_base.den &&
           ffmpegdec->format.video.fps_base ==
-              ffmpegdec->context->frame_rate_base)
+              ffmpegdec->context->time_base.num &&
+	  ffmpegdec->format.video.pix_fmt == ffmpegdec->context->pix_fmt)
         return TRUE;
       GST_DEBUG ("Renegotiating video from %dx%d@%d/%dfps to %dx%d@%d/%dfps",
           ffmpegdec->format.video.width, ffmpegdec->format.video.height,
           ffmpegdec->format.video.fps, ffmpegdec->format.video.fps_base,
           ffmpegdec->context->width, ffmpegdec->context->height,
-          ffmpegdec->context->frame_rate, ffmpegdec->context->frame_rate_base);
+          ffmpegdec->context->time_base.den,
+          ffmpegdec->context->time_base.num);
       ffmpegdec->format.video.width = ffmpegdec->context->width;
       ffmpegdec->format.video.height = ffmpegdec->context->height;
-      ffmpegdec->format.video.fps = ffmpegdec->context->frame_rate;
-      ffmpegdec->format.video.fps_base = ffmpegdec->context->frame_rate_base;
+      ffmpegdec->format.video.fps = ffmpegdec->context->time_base.den;
+      ffmpegdec->format.video.fps_base = ffmpegdec->context->time_base.num;
+      ffmpegdec->format.video.pix_fmt = ffmpegdec->context->pix_fmt;
       break;
     case CODEC_TYPE_AUDIO:
       if (ffmpegdec->format.audio.samplerate ==
@@ -619,59 +670,85 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
   GstFFMpegDecClass *oclass =
       (GstFFMpegDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
   GstBuffer *outbuf = NULL;
-  gint have_data, len = 0;
-
+  gint have_data = 0, len = 0;
+  
   ffmpegdec->context->frame_number++;
 
   switch (oclass->in_plugin->type) {
     case CODEC_TYPE_VIDEO:
       ffmpegdec->picture->pict_type = -1; /* in case we skip frames */
+
+      ffmpegdec->context->opaque = ffmpegdec;
+      
       len = avcodec_decode_video (ffmpegdec->context,
           ffmpegdec->picture, &have_data, data, size);
       GST_DEBUG_OBJECT (ffmpegdec,
           "Decode video: len=%d, have_data=%d", len, have_data);
 
-      if (len >= 0 && have_data > 0) {
+      if (ffmpegdec->waiting_for_key &&
+          ffmpegdec->picture->pict_type != FF_I_TYPE) {
+        have_data = 0;
+      } else if (len >= 0 && have_data > 0) {
         /* libavcodec constantly crashes on stupid buffer allocation
          * errors inside. This drives me crazy, so we let it allocate
          * it's own buffers and copy to our own buffer afterwards... */
-        AVPicture pic;
-        gint fsize = gst_ffmpeg_avpicture_get_size (ffmpegdec->context->pix_fmt,
+
+	if (ffmpegdec->picture->opaque != NULL) {
+	  outbuf = (GstBuffer *) ffmpegdec->picture->opaque;
+          if (outbuf == ffmpegdec->last_buffer)
+            ffmpegdec->last_buffer = NULL;
+	} else {
+	  AVPicture pic;
+	  gint fsize = gst_ffmpeg_avpicture_get_size (ffmpegdec->context->pix_fmt,
             ffmpegdec->context->width, ffmpegdec->context->height);
 
-        outbuf = gst_buffer_new_and_alloc (fsize);
+	  if (!gst_ffmpegdec_negotiate (ffmpegdec))
+	    return -1;	
+	  
+	  outbuf = gst_pad_alloc_buffer (ffmpegdec->srcpad, GST_BUFFER_OFFSET_NONE, fsize);
 
-        /* original ffmpeg code does not handle odd sizes correctly.
-         * This patched up version does */
-        gst_ffmpeg_avpicture_fill (&pic, GST_BUFFER_DATA (outbuf),
-            ffmpegdec->context->pix_fmt,
-            ffmpegdec->context->width, ffmpegdec->context->height);
+	  /* original ffmpeg code does not handle odd sizes correctly.
+	   * This patched up version does */
+	  gst_ffmpeg_avpicture_fill (&pic, GST_BUFFER_DATA (outbuf),
+				     ffmpegdec->context->pix_fmt,
+				     ffmpegdec->context->width, ffmpegdec->context->height);
 
-        /* the original convert function did not do the right thing, this
-         * is a patched up version that adjust widht/height so that the
-         * ffmpeg one works correctly. */
-        gst_ffmpeg_img_convert (&pic, ffmpegdec->context->pix_fmt,
-            (AVPicture *) ffmpegdec->picture,
-            ffmpegdec->context->pix_fmt,
-            ffmpegdec->context->width, 
-            ffmpegdec->context->height);
+	  /* the original convert function did not do the right thing, this
+	   * is a patched up version that adjust widht/height so that the
+	   * ffmpeg one works correctly. */
+	  gst_ffmpeg_img_convert (&pic, ffmpegdec->context->pix_fmt,
+				  (AVPicture *) ffmpegdec->picture,
+				  ffmpegdec->context->pix_fmt,
+				  ffmpegdec->context->width, 
+				  ffmpegdec->context->height);
+	}	
+	
+	ffmpegdec->waiting_for_key = FALSE;
 
         /* note that ffmpeg sometimes gets the FPS wrong.
          * For B-frame containing movies, we get all pictures delayed
          * except for the I frames, so we synchronize only on I frames
          * and keep an internal counter based on FPS for the others. */
-        if ((ffmpegdec->picture->pict_type == FF_I_TYPE ||
-             !GST_CLOCK_TIME_IS_VALID (ffmpegdec->next_ts)) &&
-            GST_CLOCK_TIME_IS_VALID (*in_ts)) {
+        if (!(oclass->in_plugin->capabilities & CODEC_CAP_DELAY) ||
+	    ((ffmpegdec->picture->pict_type == FF_I_TYPE ||
+              !GST_CLOCK_TIME_IS_VALID (ffmpegdec->next_ts)) &&
+             GST_CLOCK_TIME_IS_VALID (*in_ts))) {
           ffmpegdec->next_ts = *in_ts;
+	  *in_ts = GST_CLOCK_TIME_NONE;
         }
 
+	if (ffmpegdec->picture->key_frame) {
+	  GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
+	} else {
+	  GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_DELTA_UNIT);
+	}
+		
         GST_BUFFER_TIMESTAMP (outbuf) = ffmpegdec->next_ts;
-        if (ffmpegdec->context->frame_rate_base != 0 &&
-            ffmpegdec->context->frame_rate != 0) {
+        if (ffmpegdec->context->time_base.num != 0 &&
+            ffmpegdec->context->time_base.den != 0) {
           GST_BUFFER_DURATION (outbuf) = GST_SECOND *
-              ffmpegdec->context->frame_rate_base /
-              ffmpegdec->context->frame_rate;
+              ffmpegdec->context->time_base.num /
+              ffmpegdec->context->time_base.den;
 
           /* Take repeat_pict into account */
           GST_BUFFER_DURATION (outbuf) += GST_BUFFER_DURATION (outbuf)
@@ -681,19 +758,21 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
         } else {
           ffmpegdec->next_ts = GST_CLOCK_TIME_NONE;
         }
-      } else if (ffmpegdec->picture->pict_type != -1) {
+      } else if (ffmpegdec->picture->pict_type != -1 &&
+		 oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
         /* update time for skip-frame */
         if ((ffmpegdec->picture->pict_type == FF_I_TYPE ||
              !GST_CLOCK_TIME_IS_VALID (ffmpegdec->next_ts)) &&
             GST_CLOCK_TIME_IS_VALID (*in_ts)) {
           ffmpegdec->next_ts = *in_ts;
+	  *in_ts = GST_CLOCK_TIME_NONE;
         }
         
-        if (ffmpegdec->context->frame_rate_base != 0 &&
-            ffmpegdec->context->frame_rate != 0) {
+        if (ffmpegdec->context->time_base.num != 0 &&
+            ffmpegdec->context->time_base.den != 0) {
           guint64 dur = GST_SECOND *  
-            ffmpegdec->context->frame_rate_base /
-            ffmpegdec->context->frame_rate;
+            ffmpegdec->context->time_base.num /
+            ffmpegdec->context->time_base.den;
 
           /* Take repeat_pict into account */
           dur += dur * ffmpegdec->picture->repeat_pict / 2;
@@ -706,13 +785,23 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
       break;
 
     case CODEC_TYPE_AUDIO:
-      outbuf = gst_buffer_new_and_alloc (AVCODEC_MAX_AUDIO_FRAME_SIZE);
+      if (!ffmpegdec->last_buffer)
+        outbuf = gst_buffer_new_and_alloc (AVCODEC_MAX_AUDIO_FRAME_SIZE);
+      else {
+        outbuf = ffmpegdec->last_buffer;
+        ffmpegdec->last_buffer = NULL;
+      }
       len = avcodec_decode_audio (ffmpegdec->context,
           (int16_t *) GST_BUFFER_DATA (outbuf), &have_data, data, size);
       GST_DEBUG_OBJECT (ffmpegdec,
           "Decode audio: len=%d, have_data=%d", len, have_data);
 
       if (len >= 0 && have_data > 0) {
+	if (!gst_ffmpegdec_negotiate (ffmpegdec)) {
+	  gst_buffer_unref (outbuf);
+	  return -1;
+	}
+
         GST_BUFFER_SIZE (outbuf) = have_data;
         if (GST_CLOCK_TIME_IS_VALID (*in_ts)) {
           ffmpegdec->next_ts = *in_ts;
@@ -722,7 +811,11 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
             (2 * ffmpegdec->context->channels *
             ffmpegdec->context->sample_rate);
         ffmpegdec->next_ts += GST_BUFFER_DURATION (outbuf);
-        *in_ts += GST_BUFFER_DURATION (outbuf);
+        if (GST_CLOCK_TIME_IS_VALID (*in_ts))
+          *in_ts += GST_BUFFER_DURATION (outbuf);
+      } else if (len > 0 && have_data == 0) {
+        /* cache output, because it may be used for caching (in-place) */
+        ffmpegdec->last_buffer = outbuf;
       } else {
         gst_buffer_unref (outbuf);
       }
@@ -731,7 +824,7 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
       g_assert (0);
       break;
   }
-
+  
   if (len < 0 || have_data < 0) {
     GST_ERROR_OBJECT (ffmpegdec,
         "ffdec_%s: decoding error (len: %d, have_data: %d)",
@@ -745,16 +838,12 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
     /* this is where I lost my last clue on ffmpeg... */
     *got_data = 1; //(ffmpegdec->pctx || have_data) ? 1 : 0;
   }
-
+  
   if (have_data) {
+
     GST_DEBUG_OBJECT (ffmpegdec, "Decoded data, now pushing (%"
         GST_TIME_FORMAT ")", GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
 
-    if (!gst_ffmpegdec_negotiate (ffmpegdec)) {
-      gst_buffer_unref (outbuf);
-      return -1;
-    }
-
     if (GST_PAD_IS_USABLE (ffmpegdec->srcpad))
       gst_pad_push (ffmpegdec->srcpad, GST_DATA (outbuf));
     else
@@ -767,21 +856,24 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
 static void
 gst_ffmpegdec_handle_event (GstFFMpegDec * ffmpegdec, GstEvent * event)
 {
+  GstFFMpegDecClass *oclass =
+      (GstFFMpegDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
+
   GST_DEBUG_OBJECT (ffmpegdec,
       "Handling event of type %d", GST_EVENT_TYPE (event));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_EOS: {
-      gint have_data, len;
-
-      do {
-        len = gst_ffmpegdec_frame (ffmpegdec, NULL, 0, &have_data,
-            &ffmpegdec->next_ts);
-        if (len < 0 || have_data == 0)
-          break;
-      } while (1);
+    case GST_EVENT_EOS:
+      if (oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
+        gint have_data, len, try = 0;
+        do {
+          len = gst_ffmpegdec_frame (ffmpegdec, NULL, 0, &have_data,
+              &ffmpegdec->next_ts);
+          if (len < 0 || have_data == 0)
+            break;
+        } while (try++ < 10);
+      }
       goto forward;
-    }
     case GST_EVENT_FLUSH:
       if (ffmpegdec->opened) {
         avcodec_flush_buffers (ffmpegdec->context);
@@ -812,6 +904,12 @@ gst_ffmpegdec_handle_event (GstFFMpegDec * ffmpegdec, GstEvent * event)
       }
       if (ffmpegdec->opened) {
         avcodec_flush_buffers (ffmpegdec->context);
+
+        if (ffmpegdec->context->codec_id == CODEC_ID_MPEG2VIDEO ||
+            ffmpegdec->context->codec_id == CODEC_ID_MPEG4 ||
+            ffmpegdec->context->codec_id == CODEC_ID_H264) {
+          ffmpegdec->waiting_for_key = TRUE;
+        }
       }
       /* fall-through */
     }
@@ -878,11 +976,9 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
     /* parse, if at all possible */
     if (ffmpegdec->pctx) {
       gint res;
-      gint64 ffpts = AV_NOPTS_VALUE;
+      gint64 ffpts;
 
-      if (GST_CLOCK_TIME_IS_VALID (in_ts))
-	ffpts = in_ts / (GST_SECOND / AV_TIME_BASE);
-    
+      ffpts = gst_ffmpeg_time_gst_to_ff (in_ts, ffmpegdec->context->time_base);
       res = av_parser_parse (ffmpegdec->pctx, ffmpegdec->context,
           &data, &size, bdata, bsize,
           ffpts, ffpts);
@@ -890,9 +986,8 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
       GST_DEBUG_OBJECT (ffmpegdec, "Parsed video frame, res=%d, size=%d",
           res, size);
       
-      if (ffmpegdec->pctx->pts != AV_NOPTS_VALUE)
-        in_ts = ffmpegdec->pctx->pts * (GST_SECOND / AV_TIME_BASE);
-
+      in_ts = gst_ffmpeg_time_ff_to_gst (ffmpegdec->pctx->pts,
+          ffmpegdec->context->time_base);
       if (res == 0 || size == 0)
         break;
       else {
@@ -924,6 +1019,8 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
 
     ffmpegdec->pcache = gst_buffer_create_sub (inbuf,
         GST_BUFFER_SIZE (inbuf) - bsize, bsize);
+  } else if (bsize > 0) {
+    GST_DEBUG_OBJECT (ffmpegdec, "Dropping %d bytes of data", bsize);
   }
   gst_buffer_unref (inbuf);
 }
@@ -937,6 +1034,10 @@ gst_ffmpegdec_change_state (GstElement * element)
   switch (transition) {
     case GST_STATE_PAUSED_TO_READY:
       gst_ffmpegdec_close (ffmpegdec);
+      if (ffmpegdec->last_buffer != NULL) {
+	gst_buffer_unref (ffmpegdec->last_buffer);
+        ffmpegdec->last_buffer = NULL;
+      }
       break;
   }
 
@@ -1026,8 +1127,11 @@ gst_ffmpegdec_register (GstPlugin * plugin)
     }
 
     /* name */
-    if (!gst_ffmpeg_get_codecid_longname (in_plugin->id))
+    if (!gst_ffmpeg_get_codecid_longname (in_plugin->id)) {
+      g_warning ("Add decoder %s (%d) please",
+          in_plugin->name, in_plugin->id);
       goto next;
+    }
 
     /* first make sure we've got a supported type */
     sinkcaps = gst_ffmpeg_codecid_to_caps (in_plugin->id, NULL, FALSE);
@@ -1064,24 +1168,25 @@ gst_ffmpegdec_register (GstPlugin * plugin)
     /* (Ronald) MPEG-4 gets a higher priority because it has been well-
      * tested and by far outperforms divxdec/xviddec - so we prefer it.
      * msmpeg4v3 same, as it outperforms divxdec for divx3 playback.
-     * H263 has the same mimetype as H263I and since H263 works for the
-     * few streams that I've tried (see, e.g., #155163), I'll use that
-     * and use rank=none for H263I for now, until I know what the diff
-     * is. */
+     * VC1/WMV3 are not working and thus unpreferred for now. */
     switch (in_plugin->id) {
       case CODEC_ID_MPEG4:
       case CODEC_ID_MSMPEG4V3:
+      case CODEC_ID_H264:
         rank = GST_RANK_PRIMARY;
         break;
       default:
         rank = GST_RANK_MARGINAL;
         break;
+      case CODEC_ID_WMV3:
+      case CODEC_ID_VC9:
       /* what's that? */
       case CODEC_ID_SP5X:
         rank = GST_RANK_NONE;
         break;
     }
     if (!gst_element_register (plugin, type_name, rank, type)) {
+      g_warning ("Failed to register %s", type_name);
       g_free (type_name);
       return FALSE;
     }
