@@ -316,8 +316,8 @@ gst_ffmpegenc_getcaps (GstPad * pad)
   /* set some default properties */
   ctx->width = 384;
   ctx->height = 288;
-  ctx->frame_rate_base = DEFAULT_FRAME_RATE_BASE;
-  ctx->frame_rate = 25 * DEFAULT_FRAME_RATE_BASE;
+  ctx->time_base.num = DEFAULT_FRAME_RATE_BASE;
+  ctx->time_base.den = 25 * DEFAULT_FRAME_RATE_BASE;
   ctx->bit_rate = 350 * 1000;
   /* makes it silent */
   ctx->strict_std_compliance = -1;
@@ -331,9 +331,10 @@ gst_ffmpegenc_getcaps (GstPad * pad)
         caps = gst_caps_new_empty ();
       gst_caps_append (caps,
           gst_ffmpeg_codectype_to_caps (oclass->in_plugin->type, ctx));
+      avcodec_close (ctx);
     }
-    /* FIXME: ffmpeg likes to crash on this */
-    avcodec_close (ctx);
+    if (ctx->priv_data)
+      avcodec_close (ctx);
   }
   av_free (ctx);
 
@@ -384,6 +385,10 @@ gst_ffmpegenc_link (GstPad * pad, const GstCaps * caps)
   /* fetch pix_fmt and so on */
   gst_ffmpeg_caps_with_codectype (oclass->in_plugin->type,
       caps, ffmpegenc->context);
+  if (!ffmpegenc->context->time_base.den) {
+    ffmpegenc->context->time_base.den = 25;
+    ffmpegenc->context->time_base.num = 1;
+  }
 
   pix_fmt = ffmpegenc->context->pix_fmt;
 
@@ -459,26 +464,33 @@ gst_ffmpegenc_chain_video (GstPad * pad, GstData * _data)
   GstBuffer *inbuf = GST_BUFFER (_data), *outbuf;
   GstFFMpegEncClass *oclass =
       (GstFFMpegEncClass *) (G_OBJECT_GET_CLASS (ffmpegenc));
-  gint ret_size = 0;
+  gint ret_size = 0, frame_size;
+  const AVRational bq = { 1, 1000000000 };
 
   /* FIXME: events (discont (flush!) and eos (close down) etc.) */
 
-  outbuf = gst_buffer_new_and_alloc (ffmpegenc->buffer_size);
+  GST_DEBUG_OBJECT (ffmpegenc,
+      "Received buffer of time %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (inbuf)));
 
-  gst_ffmpeg_avpicture_fill ((AVPicture *) ffmpegenc->picture,
+  frame_size = gst_ffmpeg_avpicture_fill ((AVPicture *) ffmpegenc->picture,
       GST_BUFFER_DATA (inbuf),
       ffmpegenc->context->pix_fmt,
       ffmpegenc->context->width, ffmpegenc->context->height);
+  g_return_if_fail (frame_size == GST_BUFFER_SIZE (inbuf));
 
-  ffmpegenc->picture->pts = GST_BUFFER_TIMESTAMP (inbuf) / 1000;
+  ffmpegenc->picture->pts =
+      gst_ffmpeg_time_gst_to_ff (GST_BUFFER_TIMESTAMP (inbuf),
+          ffmpegenc->context->time_base);
 
+  outbuf = gst_buffer_new_and_alloc (ffmpegenc->buffer_size);
   ret_size = avcodec_encode_video (ffmpegenc->context,
       GST_BUFFER_DATA (outbuf),
       GST_BUFFER_MAXSIZE (outbuf), ffmpegenc->picture);
 
   if (ret_size < 0) {
-    GST_ELEMENT_ERROR (ffmpegenc, LIBRARY, ENCODE, (NULL),
-        ("ffenc_%s: failed to encode buffer", oclass->in_plugin->name));
+    GST_ERROR_OBJECT (ffmpegenc,
+        "ffenc_%s: failed to encode buffer", oclass->in_plugin->name);
     gst_buffer_unref (inbuf);
     gst_buffer_unref (outbuf);
     return;
@@ -487,6 +499,10 @@ gst_ffmpegenc_chain_video (GstPad * pad, GstData * _data)
   GST_BUFFER_SIZE (outbuf) = ret_size;
   GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (inbuf);
   GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (inbuf);
+  if (ffmpegenc->context->coded_frame->key_frame)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
+  else
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_DELTA_UNIT);
   gst_pad_push (ffmpegenc->srcpad, GST_DATA (outbuf));
 
   gst_buffer_unref (inbuf);
@@ -498,8 +514,6 @@ gst_ffmpegenc_chain_audio (GstPad * pad, GstData * _data)
   GstBuffer *inbuf = GST_BUFFER (_data);
   GstBuffer *outbuf = NULL, *subbuf;
   GstFFMpegEnc *ffmpegenc = (GstFFMpegEnc *) (gst_pad_get_parent (pad));
-  GstFFMpegEncClass *oclass =
-      (GstFFMpegEncClass *) (G_OBJECT_GET_CLASS (ffmpegenc));
   gint size, ret_size = 0, in_size, frame_size;
 
   size = GST_BUFFER_SIZE (inbuf);
@@ -512,6 +526,10 @@ gst_ffmpegenc_chain_audio (GstPad * pad, GstData * _data)
   if (ffmpegenc->cache)
     in_size += GST_BUFFER_SIZE (ffmpegenc->cache);
 
+  GST_DEBUG_OBJECT (ffmpegenc,
+      "Received buffer of time %" GST_TIME_FORMAT " and size %d (cache: %d)",
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (inbuf)), size, in_size - size);
+
   while (1) {
     /* do we have enough data for one frame? */
     if (in_size / (2 * ffmpegenc->context->channels) <
@@ -564,7 +582,7 @@ gst_ffmpegenc_chain_audio (GstPad * pad, GstData * _data)
         GST_BUFFER_DATA (subbuf));
 
     if (ret_size < 0) {
-      g_warning ("ffenc_%s: failed to encode buffer", oclass->in_plugin->name);
+      GST_ERROR_OBJECT (ffmpegenc, "Failed to encode buffer");
       gst_buffer_unref (inbuf);
       gst_buffer_unref (outbuf);
       gst_buffer_unref (subbuf);
@@ -704,8 +722,11 @@ gst_ffmpegenc_register (GstPlugin * plugin)
     }
 
     /* name */
-    if (!gst_ffmpeg_get_codecid_longname (in_plugin->id))
+    if (!gst_ffmpeg_get_codecid_longname (in_plugin->id)) {
+      g_warning ("Add encoder %s (%d) please",
+          in_plugin->name, in_plugin->id);
       goto next;
+    }
 
     /* first make sure we've got a supported type */
     srccaps = gst_ffmpeg_codecid_to_caps (in_plugin->id, NULL, TRUE);
