@@ -45,6 +45,8 @@ struct _GstFFMpegMux
   AVFormatContext *context;
   gboolean opened;
 
+  GstTagList *tags;
+
   GstPad *sinkpads[MAX_STREAMS];
   gint videopads, audiopads;
   GstBuffer *bufferqueue[MAX_STREAMS];
@@ -95,7 +97,7 @@ static GHashTable *global_plugins;
 static void gst_ffmpegmux_class_init (GstFFMpegMuxClass * klass);
 static void gst_ffmpegmux_base_init (GstFFMpegMuxClass * klass);
 static void gst_ffmpegmux_init (GstFFMpegMux * ffmpegmux);
-static void gst_ffmpegmux_dispose (GObject * object);
+static void gst_ffmpegmux_finalize (GObject * object);
 
 static GstPadLinkReturn
 gst_ffmpegmux_connect (GstPad * pad, const GstCaps * caps);
@@ -165,7 +167,7 @@ gst_ffmpegmux_class_init (GstFFMpegMuxClass * klass)
 
   gstelement_class->request_new_pad = gst_ffmpegmux_request_new_pad;
   gstelement_class->change_state = gst_ffmpegmux_change_state;
-  gobject_class->dispose = gst_ffmpegmux_dispose;
+  gobject_class->finalize = gst_ffmpegmux_finalize;
 }
 
 static void
@@ -189,19 +191,19 @@ gst_ffmpegmux_init (GstFFMpegMux * ffmpegmux)
 
   ffmpegmux->videopads = 0;
   ffmpegmux->audiopads = 0;
+
+  ffmpegmux->tags = NULL;
 }
 
 static void
-gst_ffmpegmux_dispose (GObject * object)
+gst_ffmpegmux_finalize (GObject * object)
 {
   GstFFMpegMux *ffmpegmux = (GstFFMpegMux *) object;
 
-  if (ffmpegmux->opened) {
-    url_fclose (&ffmpegmux->context->pb);
-    ffmpegmux->opened = FALSE;
-  }
-
   g_free (ffmpegmux->context);
+
+  if (G_OBJECT_CLASS (parent_class)->finalize)
+    G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstPad *
@@ -245,11 +247,13 @@ gst_ffmpegmux_request_new_pad (GstElement * element,
 
   /* AVStream needs to be created */
   st = av_new_stream (ffmpegmux->context, padnum);
-  st->codec.codec_type = type;
-  st->codec.codec_id = CODEC_ID_NONE;   /* this is a check afterwards */
+  st->codec->codec_type = type;
+  st->codec->codec_id = CODEC_ID_NONE;   /* this is a check afterwards */
   st->stream_copy = 1;          /* we're not the actual encoder */
-  st->codec.bit_rate = bitrate;
-  st->codec.frame_size = framesize;
+  st->codec->bit_rate = bitrate;
+  st->codec->frame_size = framesize;
+  st->time_base.den = GST_SECOND;
+  st->time_base.num = 1;
   /* we fill in codec during capsnego */
 
   /* we love debug output (c) (tm) (r) */
@@ -269,7 +273,6 @@ gst_ffmpegmux_connect (GstPad * pad, const GstCaps * caps)
 
   /*g_return_val_if_fail (ffmpegmux->opened == FALSE,
      GST_PAD_LINK_REFUSED); */
-
   for (i = 0; i < ffmpegmux->context->nb_streams; i++) {
     if (pad == ffmpegmux->sinkpads[i]) {
       break;
@@ -283,7 +286,7 @@ gst_ffmpegmux_connect (GstPad * pad, const GstCaps * caps)
 
   /* for the format-specific guesses, we'll go to
    * our famous codec mapper */
-  if (gst_ffmpeg_caps_to_codecid (caps, &st->codec) != CODEC_ID_NONE) {
+  if (gst_ffmpeg_caps_to_codecid (caps, st->codec) != CODEC_ID_NONE) {
     ffmpegmux->eos[i] = FALSE;
     return GST_PAD_LINK_OK;
   }
@@ -318,6 +321,16 @@ gst_ffmpegmux_loop (GstElement * element)
             ffmpegmux->eos[i] = TRUE;
             gst_event_unref (event);
             break;
+          case GST_EVENT_TAG:
+            if (ffmpegmux->tags) {
+              gst_tag_list_insert (ffmpegmux->tags,
+                  gst_event_tag_get_list (event), GST_TAG_MERGE_PREPEND);
+            } else {
+              ffmpegmux->tags =
+                  gst_tag_list_copy (gst_event_tag_get_list (event));
+            }
+            gst_event_unref (event);
+            break;
           default:
             gst_pad_event_default (pad, event);
             break;
@@ -330,29 +343,89 @@ gst_ffmpegmux_loop (GstElement * element)
 
   /* open "file" (gstreamer protocol to next element) */
   if (!ffmpegmux->opened) {
+    const GstTagList *iface_tags;
+    int open_flags = URL_WRONLY;
+
     /* we do need all streams to have started capsnego,
      * or things will go horribly wrong */
     for (i = 0; i < ffmpegmux->context->nb_streams; i++) {
       AVStream *st = ffmpegmux->context->streams[i];
 
       /* check whether the pad has successfully completed capsnego */
-      if (st->codec.codec_id == CODEC_ID_NONE) {
+      if (st->codec->codec_id == CODEC_ID_NONE) {
         GST_ELEMENT_ERROR (element, CORE, NEGOTIATION, (NULL),
             ("no caps set on stream %d (%s)", i,
-                (st->codec.codec_type == CODEC_TYPE_VIDEO) ?
+                (st->codec->codec_type == CODEC_TYPE_VIDEO) ?
                 "video" : "audio"));
         return;
       }
+      if (st->codec->codec_type == CODEC_TYPE_AUDIO) {
+        st->codec->frame_size =
+            st->codec->sample_rate *
+            GST_BUFFER_DURATION (ffmpegmux->bufferqueue[i]) / GST_SECOND;
+      }
     }
 
+    /* tags */
+    iface_tags = gst_tag_setter_get_list (GST_TAG_SETTER (ffmpegmux));
+    if (ffmpegmux->tags || iface_tags) {
+      GstTagList *tags;
+      gint i;
+      gchar *s;
+
+      if (iface_tags && ffmpegmux->tags) {
+        gst_tag_list_merge (iface_tags, ffmpegmux->tags,
+            GST_TAG_MERGE_APPEND);
+      } else if (iface_tags) {
+        tags = gst_tag_list_copy (iface_tags);
+      } else {
+        tags = gst_tag_list_copy (ffmpegmux->tags);
+      }
+
+      /* get the interesting ones */
+      if (gst_tag_list_get_string (tags, GST_TAG_TITLE, &s)) {
+        strncpy (ffmpegmux->context->title, s,
+            sizeof (ffmpegmux->context->title));
+      }
+      if (gst_tag_list_get_string (tags, GST_TAG_ARTIST, &s)) {
+        strncpy (ffmpegmux->context->author, s,
+            sizeof (ffmpegmux->context->author));
+      }
+      if (gst_tag_list_get_string (tags, GST_TAG_COPYRIGHT, &s)) {
+        strncpy (ffmpegmux->context->copyright, s,
+            sizeof (ffmpegmux->context->copyright));
+      }
+      if (gst_tag_list_get_string (tags, GST_TAG_COMMENT, &s)) {
+        strncpy (ffmpegmux->context->comment, s,
+            sizeof (ffmpegmux->context->comment));
+      }
+      if (gst_tag_list_get_string (tags, GST_TAG_ALBUM, &s)) {
+        strncpy (ffmpegmux->context->album, s,
+            sizeof (ffmpegmux->context->album));
+      }
+      if (gst_tag_list_get_string (tags, GST_TAG_GENRE, &s)) {
+        strncpy (ffmpegmux->context->genre, s,
+            sizeof (ffmpegmux->context->genre));
+      }
+      if (gst_tag_list_get_uint (tags, GST_TAG_TRACK_NUMBER, &i)) {
+        ffmpegmux->context->track = i;
+      }
+      gst_tag_list_free (tags);
+    }
+
+	/* set the streamheader flag for gstffmpegprotocol if codec supports it */
+	if (!strcmp (ffmpegmux->context->oformat->name, "flv") ) {
+		open_flags |= GST_FFMPEG_URL_STREAMHEADER;
+	}
+
     if (url_fopen (&ffmpegmux->context->pb,
-            ffmpegmux->context->filename, URL_WRONLY) < 0) {
+            ffmpegmux->context->filename, open_flags) < 0) {
       GST_ELEMENT_ERROR (element, LIBRARY, TOO_LAZY, (NULL),
           ("Failed to open stream context in ffmux"));
       return;
     }
 
-    if (av_set_parameters (ffmpegmux->context, NULL)) {
+    if (av_set_parameters (ffmpegmux->context, NULL) < 0) {
       GST_ELEMENT_ERROR (element, LIBRARY, INIT, (NULL),
           ("Failed to initialize muxer"));
       return;
@@ -362,7 +435,14 @@ gst_ffmpegmux_loop (GstElement * element)
     ffmpegmux->opened = TRUE;
 
     /* now open the mux format */
-    av_write_header (ffmpegmux->context);
+    if (av_write_header (ffmpegmux->context) < 0) {
+      GST_ELEMENT_ERROR (element, LIBRARY, SETTINGS, (NULL),
+          ("Failed to write file header - check codec settings"));
+      return;
+    }
+	
+    /* flush the header so it will be used as streamheader */
+    put_flush_packet (&ffmpegmux->context->pb);
   }
 
   /* take the one with earliest timestamp,
@@ -392,21 +472,26 @@ gst_ffmpegmux_loop (GstElement * element)
   if (bufnum >= 0) {
     GstBuffer *buf;
     AVPacket pkt;
+    AVRational bq = { 1, 1000000000 };
 
     /* push out current buffer */
     buf = ffmpegmux->bufferqueue[bufnum];
     ffmpegmux->bufferqueue[bufnum] = NULL;
 
-    ffmpegmux->context->streams[bufnum]->codec.frame_number++;
+    ffmpegmux->context->streams[bufnum]->codec->frame_number++;
 
     /* set time */
-    pkt.pts = GST_BUFFER_TIMESTAMP (buf) * AV_TIME_BASE / GST_SECOND;
+    pkt.pts = gst_ffmpeg_time_gst_to_ff (GST_BUFFER_TIMESTAMP (buf),
+        ffmpegmux->context->streams[bufnum]->time_base);
+    pkt.dts = pkt.pts;
     pkt.data = GST_BUFFER_DATA (buf);
     pkt.size = GST_BUFFER_SIZE (buf);
     pkt.stream_index = bufnum;
     pkt.flags = 0;
-    if (GST_BUFFER_FLAGS (buf) & GST_BUFFER_KEY_UNIT)
-       pkt.flags |= PKT_FLAG_KEY;
+
+    if (!(GST_BUFFER_FLAGS (buf) & GST_BUFFER_DELTA_UNIT))
+      pkt.flags |= PKT_FLAG_KEY;    
+
     if (GST_BUFFER_DURATION_IS_VALID (buf))
       pkt.duration = GST_BUFFER_DURATION (buf) * AV_TIME_BASE / GST_SECOND;
     else
@@ -416,8 +501,8 @@ gst_ffmpegmux_loop (GstElement * element)
   } else {
     /* close down */
     av_write_trailer (ffmpegmux->context);
-    url_fclose (&ffmpegmux->context->pb);
     ffmpegmux->opened = FALSE;
+    url_fclose (&ffmpegmux->context->pb);
     gst_element_set_eos (element);
   }
 }
@@ -430,9 +515,13 @@ gst_ffmpegmux_change_state (GstElement * element)
 
   switch (transition) {
     case GST_STATE_PAUSED_TO_READY:
+      if (ffmpegmux->tags) {
+        gst_tag_list_free (ffmpegmux->tags);
+        ffmpegmux->tags = NULL;
+      }
       if (ffmpegmux->opened) {
-        url_fclose (&ffmpegmux->context->pb);
         ffmpegmux->opened = FALSE;
+        url_fclose (&ffmpegmux->context->pb);
       }
       break;
   }
@@ -443,6 +532,20 @@ gst_ffmpegmux_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+GstCaps *
+gst_ffmpegmux_get_id_caps (enum CodecID * id_list)
+{
+  GstCaps *caps, *t;
+  gint i;
+
+  caps = gst_caps_new_empty ();
+  for (i = 0; id_list[i] != CODEC_ID_NONE; i++) {
+    if ((t = gst_ffmpeg_codecid_to_caps (id_list[i], NULL, TRUE)))
+      gst_caps_append (caps, t);
+  }
+
+  return caps;
+}
 
 gboolean
 gst_ffmpegmux_register (GstPlugin * plugin)
@@ -458,10 +561,12 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     0,
     (GInstanceInitFunc) gst_ffmpegmux_init,
   };
+  static const GInterfaceInfo tag_setter_info = {
+      NULL, NULL, NULL
+  };
   GType type;
   AVOutputFormat *in_plugin;
   GstFFMpegMuxClassParams *params;
-  AVCodec *in_codec;
 
   in_plugin = first_oformat;
 
@@ -471,35 +576,20 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     gchar *type_name;
     gchar *p;
     GstCaps *srccaps, *audiosinkcaps, *videosinkcaps;
+    enum CodecID *video_ids = NULL, *audio_ids = NULL;
 
     /* Try to find the caps that belongs here */
     srccaps = gst_ffmpeg_formatid_to_caps (in_plugin->name);
     if (!srccaps) {
       goto next;
     }
-    /* This is a bit ugly, but we just take all formats
-     * for the pad template. We'll get an exact match
-     * when we open the stream */
-    audiosinkcaps = gst_caps_new_empty ();
-    videosinkcaps = gst_caps_new_empty ();
-    for (in_codec = first_avcodec; in_codec != NULL; in_codec = in_codec->next) {
-      GstCaps *temp = gst_ffmpeg_codecid_to_caps (in_codec->id, NULL, TRUE);
-
-      if (!temp) {
-        continue;
-      }
-      switch (in_codec->type) {
-        case CODEC_TYPE_VIDEO:
-          gst_caps_append (videosinkcaps, temp);
-          break;
-        case CODEC_TYPE_AUDIO:
-          gst_caps_append (audiosinkcaps, temp);
-          break;
-        default:
-          gst_caps_free (temp);
-          break;
-      }
+    if (!gst_ffmpeg_formatid_get_codecids (in_plugin->name,
+             &video_ids, &audio_ids)) {
+      gst_caps_free (srccaps);
+      goto next;
     }
+    videosinkcaps = video_ids ? gst_ffmpegmux_get_id_caps (video_ids) : NULL;
+    audiosinkcaps = audio_ids ? gst_ffmpegmux_get_id_caps (audio_ids) : NULL;
 
     /* construct the type */
     type_name = g_strdup_printf ("ffmux_%s", in_plugin->name);
@@ -515,6 +605,11 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     /* if it's already registered, drop it */
     if (g_type_from_name (type_name)) {
       g_free (type_name);
+      gst_caps_free (srccaps);
+      if (audiosinkcaps)
+        gst_caps_free (audiosinkcaps);
+      if (videosinkcaps)
+        gst_caps_free (videosinkcaps);
       goto next;
     }
 
@@ -530,8 +625,15 @@ gst_ffmpegmux_register (GstPlugin * plugin)
 
     /* create the type now */
     type = g_type_register_static (GST_TYPE_ELEMENT, type_name, &typeinfo, 0);
+    g_type_add_interface_static (type, GST_TYPE_TAG_SETTER,
+        &tag_setter_info);
     if (!gst_element_register (plugin, type_name, GST_RANK_NONE, type)) {
       g_free (type_name);
+      gst_caps_free (srccaps);
+      if (audiosinkcaps)
+        gst_caps_free (audiosinkcaps);
+      if (videosinkcaps)
+        gst_caps_free (videosinkcaps);
       return FALSE;
     }
 
