@@ -5,14 +5,14 @@
 #
 #set -x
 # Even in the 21st century some diffs are not supporting -u.
-diff -u $0 $0 > /dev/null 2>&1
+diff -u "$0" "$0" > /dev/null 2>&1
 if [ $? -eq 0 ]; then
   diff_cmd="diff -u"
 else
   diff_cmd="diff"
 fi
 
-diff -w $0 $0 > /dev/null 2>&1
+diff -w "$0" "$0" > /dev/null 2>&1
 if [ $? -eq 0 ]; then
   diff_cmd="$diff_cmd -w"
 fi
@@ -36,6 +36,8 @@ elif [ "$1" = "huffyuv" ] ; then
     do_huffyuv=y
 elif [ "$1" = "mpeg2thread" ] ; then
     do_mpeg2thread=y
+elif [ "$1" = "snow" ] ; then
+    do_snow=y
 elif [ "$1" = "libavtest" ] ; then
     do_libav=y
     logfile="$datadir/libav.regression"
@@ -73,6 +75,7 @@ else
     do_ffv1=y
     do_error=y
     do_svq1=y
+    do_snow=y
 fi
 
 
@@ -86,6 +89,7 @@ raw_dst="$datadir/out.yuv"
 raw_ref="$datadir/ref.yuv"
 pcm_src="asynth1.sw"
 pcm_dst="$datadir/out.wav"
+pcm_ref="$datadir/ref.wav"
 if [ X"`echo | md5sum 2> /dev/null`" != X ]; then
     do_md5sum() { md5sum -b $1; }
 elif [ -x /sbin/md5 ]; then
@@ -108,6 +112,8 @@ do_ffmpeg()
     do_md5sum $f >> $logfile
     if [ $f = $raw_dst ] ; then
         $tiny_psnr $f $raw_ref >> $logfile
+    elif [ $f = $pcm_dst ] ; then
+        $tiny_psnr $f $pcm_ref 2 >> $logfile
     else
         wc -c $f >> $logfile
     fi
@@ -144,6 +150,7 @@ echo "ffmpeg benchmarks" > $benchfile
 ###################################
 # generate reference for quality check
 do_ffmpeg_nocheck $raw_ref -y -f pgmyuv -i $raw_src -an -f rawvideo $raw_ref
+do_ffmpeg_nocheck $pcm_ref -y -ab 128 -ac 2 -ar 44100 -f s16le -i $pcm_src -f wav $pcm_ref
 
 ###################################
 if [ -n "$do_mpeg" ] ; then
@@ -283,7 +290,7 @@ file=${outfile}huffyuv.avi
 do_ffmpeg $file -y -f pgmyuv -i $raw_src -an -vcodec huffyuv -pix_fmt yuv422p $file
 
 # huffyuv decoding
-do_ffmpeg $raw_dst -y -i $file -f rawvideo -strict -1 $raw_dst
+do_ffmpeg $raw_dst -y -i $file -f rawvideo -strict -2 -pix_fmt yuv420p $raw_dst
 fi
 
 ###################################
@@ -360,17 +367,17 @@ fi
 if [ -n "$do_mjpeg" ] ; then
 # mjpeg
 file=${outfile}mjpeg.avi
-do_ffmpeg $file -y -qscale 10 -f pgmyuv -i $raw_src -an -vcodec mjpeg $file
+do_ffmpeg $file -y -qscale 10 -f pgmyuv -i $raw_src -an -vcodec mjpeg -pix_fmt yuvj420p $file
 
 # mjpeg decoding
-do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
+do_ffmpeg $raw_dst -y -i $file -f rawvideo -pix_fmt yuv420p $raw_dst 
 fi
 
 ###################################
 if [ -n "$do_ljpeg" ] ; then
 # ljpeg
 file=${outfile}ljpeg.avi
-do_ffmpeg $file -y -f pgmyuv -i $raw_src -an -vcodec ljpeg $file
+do_ffmpeg $file -y -f pgmyuv -i $raw_src -an -vcodec ljpeg -strict -1 $file
 
 # ljpeg decoding
 do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
@@ -430,12 +437,29 @@ fi
 if [ -n "$do_ffv1" ] ; then
 # ffv1 encoding
 file=${outfile}ffv1.avi
-do_ffmpeg $file -y -strict -1 -f pgmyuv -i $raw_src -an -vcodec ffv1 $file
+do_ffmpeg $file -y -strict -2 -f pgmyuv -i $raw_src -an -vcodec ffv1 $file
 
 # ffv1 decoding
 do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
 fi
 
+###################################
+if [ -n "$do_snow" ] ; then
+# snow encoding
+file=${outfile}snow.avi
+do_ffmpeg $file -y -strict -2 -f pgmyuv -i $raw_src -an -vcodec snow -qscale 2 $file
+
+# snow decoding
+do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
+
+# snow encoding
+file=${outfile}snow53.avi
+do_ffmpeg $file -y -strict -2 -f pgmyuv -i $raw_src -an -vcodec snow -pred 1 -qpel -4mv $file
+
+# snow decoding
+do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
+fi
+
 ###################################
 if [ -n "$do_svq1" ] ; then
 # svq1 encoding
@@ -443,7 +467,7 @@ file=${outfile}svq1.mov
 do_ffmpeg $file -y -f pgmyuv -i $raw_src -an -vcodec svq1 -qscale 3 -pix_fmt yuv410p $file
 
 # svq1 decoding
-do_ffmpeg $raw_dst -y -i $file -f rawvideo $raw_dst 
+do_ffmpeg $raw_dst -y -i $file -f rawvideo -pix_fmt yuv420p $raw_dst 
 fi
 
 ###################################
@@ -454,6 +478,7 @@ do_ffmpeg $file -y -ab 128 -ac 2 -ar 44100 -f s16le -i $pcm_src $file
 
 # mp2 decoding
 do_ffmpeg $pcm_dst -y -i $file -f wav $pcm_dst 
+$tiny_psnr $pcm_dst $pcm_ref 2 1924 >> $logfile
 fi
 
 ###################################
@@ -523,6 +548,11 @@ file=${outfile}libav.mpg
 do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src -f s16le -i $pcm_src $file
 do_ffmpeg_crc $file -i $file
 
+# mpegts
+file=${outfile}libav.ts
+do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src -f s16le -i $pcm_src $file
+do_ffmpeg_crc $file -i $file
+
 # swf (decode audio only)
 file=${outfile}libav.swf
 do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src -f s16le -i $pcm_src -acodec mp2 $file
@@ -553,8 +583,6 @@ file=${outfile}libav.dv
 do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src -f s16le -i $pcm_src -ar 48000 -r 25 -s pal -ac 2 $file
 do_ffmpeg_crc $file -i $file
 
-# XXX: need mpegts tests (add bitstreams or add output capability in ffmpeg)
-
 ####################
 # streamed images
 # mjpeg
@@ -583,7 +611,7 @@ do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src $file
 #do_ffmpeg_crc $file -i $file
 
 # yuv4mpeg
-file=${outfile}libav.yuv4mpeg
+file=${outfile}libav.y4m
 do_ffmpeg $file -t 1 -y -qscale 10 -f pgmyuv -i $raw_src $file
 #do_ffmpeg_crc $file -i $file
 
@@ -601,7 +629,7 @@ do_ffmpeg_crc $file -i $file
 
 # jpeg (we do not do md5 on image files yet)
 file=${outfile}libav%d.jpg
-$ffmpeg -t 0.5 -y -qscale 10 -f pgmyuv -i $raw_src -bitexact -dct_algo 1 -idct_algo 2 -f image2 $file
+$ffmpeg -t 0.5 -y -qscale 10 -f pgmyuv -i $raw_src -bitexact -dct_algo 1 -idct_algo 2 -pix_fmt yuvj420p -f image2 $file
 do_ffmpeg_crc $file -f image2 -i $file
 
 ####################
@@ -644,7 +672,7 @@ fi
 
 
 
-if $diff_cmd $logfile $reffile ; then
+if $diff_cmd "$logfile" "$reffile" ; then
     echo 
     echo Regression test succeeded.
     exit 0
